@@ -31,8 +31,8 @@ import (
 	. "github.com/onsi/ginkgo"
 	"github.com/onsi/ginkgo/extensions/table"
 	. "github.com/onsi/gomega"
-	"github.com/prometheus/procfs"
 
+	k8sv1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/types"
 
 	v1 "kubevirt.io/client-go/api/v1"
@@ -41,10 +41,13 @@ import (
 	"kubevirt.io/kubevirt/pkg/virt-handler/isolation"
 )
 
+const (
+	findmntByVolumeRes = "{\"filesystems\": [{\"target\":\"/%s\", \"source\":\"/dev/testvolume[%s]\", \"fstype\":\"xfs\", \"options\":\"rw,relatime,seclabel,attr2,inode64,logbufs=8,logbsize=32k,noquota\"}]}"
+)
+
 var (
 	tempDir              string
 	orgIsoDetector       = isolationDetector
-	orgProcMounts        = procMounts
 	orgDeviceBasePath    = deviceBasePath
 	orgStatCommand       = statCommand
 	orgCgroupsBasePath   = cgroupsBasePath
@@ -54,6 +57,8 @@ var (
 	orgUnMountCommand    = unmountCommand
 	orgIsMounted         = isMounted
 	orgIsBlockDevice     = isBlockDevice
+	orgFindMntByVolume   = findMntByVolume
+	orgFindMntByDevice   = findMntByDevice
 )
 
 var _ = Describe("HotplugVolume mount target records", func() {
@@ -74,6 +79,7 @@ var _ = Describe("HotplugVolume mount target records", func() {
 			podIsolationDetector: &mockIsolationDetector{},
 			mountRecords:         make(map[types.UID]*vmiMountTargetRecord),
 			mountStateDir:        tempDir,
+			hotplugDiskManager:   hotplugdisk.NewHotplugDiskWithOptions(tempDir),
 		}
 		record = &vmiMountTargetRecord{
 			MountTargetEntries: []vmiMountTargetEntry{
@@ -161,10 +167,11 @@ var _ = Describe("HotplugVolume mount target records", func() {
 
 var _ = Describe("HotplugVolume block devices", func() {
 	var (
-		m      *volumeMounter
-		err    error
-		vmi    *v1.VirtualMachineInstance
-		record *vmiMountTargetRecord
+		m             *volumeMounter
+		err           error
+		vmi           *v1.VirtualMachineInstance
+		record        *vmiMountTargetRecord
+		targetPodPath string
 	)
 
 	BeforeEach(func() {
@@ -176,11 +183,16 @@ var _ = Describe("HotplugVolume block devices", func() {
 		activePods["abcd"] = "host"
 		vmi.Status.ActivePods = activePods
 
+		targetPodPath = filepath.Join(tempDir, "abcd/volumes/kubernetes.io~empty-dir/hotplug-disks/testvolume")
+		err = os.MkdirAll(targetPodPath, 0755)
+		Expect(err).ToNot(HaveOccurred())
+
 		m = &volumeMounter{
 			podIsolationDetector: &mockIsolationDetector{},
 			mountRecords:         make(map[types.UID]*vmiMountTargetRecord),
 			mountStateDir:        tempDir,
 			skipSafetyCheck:      true,
+			hotplugDiskManager:   hotplugdisk.NewHotplugDiskWithOptions(tempDir),
 		}
 		record = &vmiMountTargetRecord{}
 
@@ -208,22 +220,33 @@ var _ = Describe("HotplugVolume block devices", func() {
 	It("isBlockVolume should determine if we have a block volume", func() {
 		err = os.RemoveAll(filepath.Join(tempDir, string(vmi.UID), "volumes"))
 		Expect(err).ToNot(HaveOccurred())
-		By("Passing empty UID, should return false")
-		res := m.isBlockVolume("")
+		vmi.Status.VolumeStatus = make([]v1.VolumeStatus, 0)
+		By("Passing invalid volume, should return false")
+		res := m.isBlockVolume(&vmi.Status, "invalid")
 		Expect(res).To(BeFalse())
-		By("Not having the volume directory, should return false")
-		res = m.isBlockVolume(vmi.UID)
+		By("Not having persistent volume info, should return false")
+		vmi.Status.VolumeStatus = append(vmi.Status.VolumeStatus, v1.VolumeStatus{
+			Name: "test",
+		})
+		res = m.isBlockVolume(&vmi.Status, "test")
 		Expect(res).To(BeFalse())
-		By("Creating the volume directory, should return true")
-		err = os.MkdirAll(filepath.Join(tempDir, string(vmi.UID), "volumes"), 0755)
-		Expect(err).ToNot(HaveOccurred())
-		isBlockDevice = func(path string) (bool, error) {
-			if strings.Contains(path, string(vmi.UID)) {
-				return true, nil
-			}
-			return false, fmt.Errorf("Not a block device")
+		By("Not having volume mode, should return false")
+		vmi.Status.VolumeStatus[0].PersistentVolumeClaimInfo = &v1.PersistentVolumeClaimInfo{}
+		res = m.isBlockVolume(&vmi.Status, "test")
+		Expect(res).To(BeFalse())
+		By("Having volume mode be filesystem, should return false")
+		fs := k8sv1.PersistentVolumeFilesystem
+		vmi.Status.VolumeStatus[0].PersistentVolumeClaimInfo = &v1.PersistentVolumeClaimInfo{
+			VolumeMode: &fs,
 		}
-		res = m.isBlockVolume(vmi.UID)
+		res = m.isBlockVolume(&vmi.Status, "test")
+		Expect(res).To(BeFalse())
+		By("Having volume mode be block, should return true")
+		block := k8sv1.PersistentVolumeBlock
+		vmi.Status.VolumeStatus[0].PersistentVolumeClaimInfo = &v1.PersistentVolumeClaimInfo{
+			VolumeMode: &block,
+		}
+		res = m.isBlockVolume(&vmi.Status, "test")
 		Expect(res).To(BeTrue())
 	})
 
@@ -232,16 +255,29 @@ var _ = Describe("HotplugVolume block devices", func() {
 		Expect(res).To(BeEquivalentTo("abcd"))
 		vmi.Status.ActivePods["abcde"] = "host"
 		res = m.findVirtlauncherUID(vmi)
+		Expect(res).To(BeEquivalentTo("abcd"))
+		vmi.Status.ActivePods["abcdef"] = "host"
+		err = os.MkdirAll(filepath.Join(tempDir, "abcdef/volumes/kubernetes.io~empty-dir/hotplug-disks"), 0755)
+		res = m.findVirtlauncherUID(vmi)
 		Expect(res).To(BeEquivalentTo(""))
 	})
 
 	It("mountBlockHotplugVolume and unmountBlockHotplugVolumes should make appropriate calls", func() {
 		blockSourcePodUID := types.UID("fghij")
-		hotplugdisk.SetKubeletPodsDirectory(tempDir)
-		targetPodPath := filepath.Join(tempDir, string(m.findVirtlauncherUID(vmi)), "volumes/kubernetes.io~empty-dir/hotplug-disks")
+		mknodCommand = func(deviceName string, major, minor int64, blockDevicePermissions string) ([]byte, error) {
+			Expect(os.MkdirAll(deviceName, 0755)).To(Succeed())
+			return []byte("Yay"), nil
+		}
+		isBlockDevice = func(path string) (bool, error) {
+			if strings.Contains(path, string(vmi.UID)) {
+				return true, nil
+			}
+			return false, fmt.Errorf("Not a block device")
+		}
+		targetPodPath := hotplugdisk.TargetPodBasePath(tempDir, m.findVirtlauncherUID(vmi))
 		err = os.MkdirAll(targetPodPath, 0755)
 		Expect(err).ToNot(HaveOccurred())
-		deviceFile := filepath.Join(tempDir, string(blockSourcePodUID), "volumes", "file")
+		deviceFile := filepath.Join(tempDir, string(blockSourcePodUID), "volumes", "testvolume", "file")
 		slicePath := "slice"
 		m.podIsolationDetector = &mockIsolationDetector{
 			slice: slicePath,
@@ -283,17 +319,17 @@ var _ = Describe("HotplugVolume block devices", func() {
 
 	It("getSourceMajorMinor should return an error if no uid", func() {
 		vmi.UID = ""
-		_, _, _, err := m.getSourceMajorMinor("fghij")
+		_, _, _, err := m.getSourceMajorMinor("fghij", "invalid")
 		Expect(err).To(HaveOccurred())
 	})
 
 	It("getSourceMajorMinor should return file if exists", func() {
-		deviceFile := filepath.Join(tempDir, "fghij", "volumes", "file")
+		deviceFile := filepath.Join(tempDir, "fghij", "volumes", "test-volume", "file")
 		err = os.MkdirAll(filepath.Dir(deviceFile), 0755)
 		Expect(err).ToNot(HaveOccurred())
 		err = ioutil.WriteFile(deviceFile, []byte("test"), 0644)
 		Expect(err).ToNot(HaveOccurred())
-		major, minor, perm, err := m.getSourceMajorMinor("fghij")
+		major, minor, perm, err := m.getSourceMajorMinor("fghij", "test-volume")
 		Expect(err).ToNot(HaveOccurred())
 		Expect(major).To(Equal(int64(6)))
 		Expect(minor).To(Equal(int64(6)))
@@ -304,7 +340,7 @@ var _ = Describe("HotplugVolume block devices", func() {
 		deviceFile := filepath.Join(tempDir, "fghij", "volumes", "file")
 		err = os.MkdirAll(filepath.Dir(deviceFile), 0755)
 		Expect(err).ToNot(HaveOccurred())
-		major, minor, perm, err := m.getSourceMajorMinor("fghij")
+		major, minor, perm, err := m.getSourceMajorMinor("fghij", "test-volume")
 		Expect(err).To(HaveOccurred())
 		Expect(major).To(Equal(int64(-1)))
 		Expect(minor).To(Equal(int64(-1)))
@@ -361,9 +397,6 @@ var _ = Describe("HotplugVolume block devices", func() {
 		table.Entry("Should not return values if stat command doesn't return block special file", func(fileName string) ([]byte, error) {
 			return []byte("245,32,0664, block file"), nil
 		}, -1, -1, "", true),
-		table.Entry("Should not return values if stat command doesn't return block special file", func(fileName string) ([]byte, error) {
-			return []byte("245,32,0664, block file"), nil
-		}, -1, -1, "", true),
 		table.Entry("Should not return values if stat command doesn't int major", func(fileName string) ([]byte, error) {
 			return []byte("kk,32,0664,block special file"), nil
 		}, -1, -1, "", true),
@@ -406,7 +439,7 @@ var _ = Describe("HotplugVolume block devices", func() {
 		Expect(err.Error()).To(ContainSubstring("no such file or directory"))
 	})
 
-	It("getTargetCgroupPath should return cgroup path", func() {
+	It("getTargetCgroupPath should return cgroup path and detect if it is not a directory", func() {
 		slicePath := "slice"
 		expectedCgroupPath := filepath.Join(tempDir, slicePath)
 		m.podIsolationDetector = &mockIsolationDetector{
@@ -566,10 +599,11 @@ var _ = Describe("HotplugVolume block devices", func() {
 
 var _ = Describe("HotplugVolume filesystem volumes", func() {
 	var (
-		m      *volumeMounter
-		err    error
-		vmi    *v1.VirtualMachineInstance
-		record *vmiMountTargetRecord
+		m             *volumeMounter
+		err           error
+		vmi           *v1.VirtualMachineInstance
+		record        *vmiMountTargetRecord
+		targetPodPath string
 	)
 
 	BeforeEach(func() {
@@ -581,28 +615,37 @@ var _ = Describe("HotplugVolume filesystem volumes", func() {
 		activePods["abcd"] = "host"
 		vmi.Status.ActivePods = activePods
 
+		targetPodPath = filepath.Join(tempDir, "abcd/volumes/kubernetes.io~empty-dir/hotplug-disks")
+		err = os.MkdirAll(targetPodPath, 0755)
+		Expect(err).ToNot(HaveOccurred())
+
 		record = &vmiMountTargetRecord{}
 
 		m = &volumeMounter{
 			podIsolationDetector: &mockIsolationDetector{},
 			mountRecords:         make(map[types.UID]*vmiMountTargetRecord),
 			mountStateDir:        tempDir,
+			hotplugDiskManager:   hotplugdisk.NewHotplugDiskWithOptions(tempDir),
 		}
 
 		deviceBasePath = func(sourceUID types.UID) string {
 			return filepath.Join(tempDir, string(sourceUID), "volumes")
 		}
-
+		isolationDetector = func(path string) isolation.PodIsolationDetector {
+			return &mockIsolationDetector{
+				pid: 1,
+			}
+		}
 	})
 
 	AfterEach(func() {
 		os.RemoveAll(tempDir)
+		findMntByVolume = orgFindMntByVolume
 		deviceBasePath = orgDeviceBasePath
 		sourcePodBasePath = orgSourcePodBasePath
 		mountCommand = orgMountCommand
 		unmountCommand = orgUnMountCommand
 		isMounted = orgIsMounted
-		procMounts = orgProcMounts
 		isolationDetector = orgIsoDetector
 	})
 
@@ -612,10 +655,13 @@ var _ = Describe("HotplugVolume filesystem volumes", func() {
 		sourcePodBasePath = func(podUID types.UID) string {
 			return path
 		}
+		findMntByVolume = func(volumeName string, pid int) ([]byte, error) {
+			return []byte(fmt.Sprintf(findmntByVolumeRes, "pvc", path)), nil
+		}
 		diskFile := filepath.Join(path, "disk.img")
 		_, err := os.Create(diskFile)
 		Expect(err).ToNot(HaveOccurred())
-		file, err := m.getSourcePodFilePath("ghfjk", vmi, "")
+		file, err := m.getSourcePodFilePath("ghfjk", vmi, "pvc")
 		Expect(err).ToNot(HaveOccurred())
 		Expect(file).To(Equal(path))
 	})
@@ -654,59 +700,41 @@ var _ = Describe("HotplugVolume filesystem volumes", func() {
 		Expect(err.Error()).To(ContainSubstring("isolation error"))
 	})
 
-	It("getSourcePodFile should return error if proc mounts returns error", func() {
+	It("getSourcePodFile should return error if find mounts returns error", func() {
 		expectedPath := filepath.Join(tempDir, "ghfjk", "volumes")
 		err = os.MkdirAll(expectedPath, 0755)
 		sourcePodBasePath = func(podUID types.UID) string {
 			return expectedPath
 		}
-		isolationDetector = func(path string) isolation.PodIsolationDetector {
-			return &mockIsolationDetector{
-				pid: 1,
-			}
-		}
-		procMounts = func(pid int) ([]*procfs.MountInfo, error) {
-			Expect(pid).To(Equal(1))
-			res := make([]*procfs.MountInfo, 0)
-			return res, fmt.Errorf("mount detection error")
+		findMntByVolume = func(volumeName string, pid int) ([]byte, error) {
+			return []byte(""), fmt.Errorf("findmnt error")
 		}
 
 		Expect(err).ToNot(HaveOccurred())
 		_, err := m.getSourcePodFilePath("ghfjk", vmi, "")
 		Expect(err).To(HaveOccurred())
-		Expect(err.Error()).To(ContainSubstring("mount detection error"))
+		Expect(err.Error()).To(ContainSubstring("findmnt error"))
 	})
 
-	It("getSourcePodFile should return the mountinfo value", func() {
+	It("getSourcePodFile should return the findmnt value", func() {
 		expectedPath := filepath.Join(tempDir, "ghfjk", "volumes")
 		err = os.MkdirAll(expectedPath, 0755)
 		sourcePodBasePath = func(podUID types.UID) string {
 			return expectedPath
 		}
-		isolationDetector = func(path string) isolation.PodIsolationDetector {
-			return &mockIsolationDetector{
-				pid: 1,
-			}
-		}
-		procMounts = func(pid int) ([]*procfs.MountInfo, error) {
-			Expect(pid).To(Equal(1))
-			res := make([]*procfs.MountInfo, 0)
-			res = append(res, &procfs.MountInfo{
-				Root:       "/test/result",
-				MountPoint: "/pvc",
-			})
-			return res, nil
+		findMntByVolume = func(volumeName string, pid int) ([]byte, error) {
+			return []byte(fmt.Sprintf(findmntByVolumeRes, "pvc", expectedPath)), nil
 		}
 
 		Expect(err).ToNot(HaveOccurred())
-		res, err := m.getSourcePodFilePath("ghfjk", vmi, "")
+		res, err := m.getSourcePodFilePath("ghfjk", vmi, "pvc")
 		Expect(err).ToNot(HaveOccurred())
-		Expect(res).To(Equal("/test/result"))
+		Expect(res).To(Equal(expectedPath))
 	})
 
 	It("should properly mount and unmount filesystem", func() {
 		sourcePodUID := "ghfjk"
-		path := filepath.Join(tempDir, sourcePodUID, "volumes")
+		path := filepath.Join(tempDir, sourcePodUID, "volumes", "disk.img")
 		err = os.MkdirAll(path, 0755)
 		sourcePodBasePath = func(podUID types.UID) string {
 			return path
@@ -714,19 +742,19 @@ var _ = Describe("HotplugVolume filesystem volumes", func() {
 		diskFile := filepath.Join(path, "disk.img")
 		_, err := os.Create(diskFile)
 		Expect(err).ToNot(HaveOccurred())
-		hotplugdisk.SetKubeletPodsDirectory(tempDir)
-		targetPodPath := filepath.Join(tempDir, string(m.findVirtlauncherUID(vmi)), "volumes/kubernetes.io~empty-dir/hotplug-disks")
-		err = os.MkdirAll(targetPodPath, 0755)
-		Expect(err).ToNot(HaveOccurred())
-		targetFilePath := filepath.Join(targetPodPath, "testvolume")
+		findMntByVolume = func(volumeName string, pid int) ([]byte, error) {
+			return []byte(fmt.Sprintf(findmntByVolumeRes, "testvolume", path)), nil
+		}
+		targetFilePath := filepath.Join(targetPodPath, "testvolume.img")
 		mountCommand = func(sourcePath, targetPath string) ([]byte, error) {
-			Expect(sourcePath).To(Equal(path))
+			Expect(sourcePath).To(Equal(diskFile))
 			Expect(targetPath).To(Equal(targetFilePath))
 			return []byte("Success"), nil
 		}
 
 		err = m.mountFileSystemHotplugVolume(vmi, "testvolume", types.UID(sourcePodUID), record)
 		Expect(err).ToNot(HaveOccurred())
+		Expect(len(record.MountTargetEntries)).To(Equal(1))
 		Expect(record.MountTargetEntries[0].TargetFile).To(Equal(targetFilePath))
 
 		unmountCommand = func(diskPath string) ([]byte, error) {
@@ -786,9 +814,10 @@ var _ = Describe("HotplugVolume filesystem volumes", func() {
 
 var _ = Describe("HotplugVolume volumes", func() {
 	var (
-		m   *volumeMounter
-		err error
-		vmi *v1.VirtualMachineInstance
+		m             *volumeMounter
+		err           error
+		vmi           *v1.VirtualMachineInstance
+		targetPodPath string
 	)
 
 	BeforeEach(func() {
@@ -800,11 +829,16 @@ var _ = Describe("HotplugVolume volumes", func() {
 		activePods["abcd"] = "host"
 		vmi.Status.ActivePods = activePods
 
+		targetPodPath = filepath.Join(tempDir, "abcd/volumes/kubernetes.io~empty-dir/hotplug-disks")
+		err = os.MkdirAll(targetPodPath, 0755)
+		Expect(err).ToNot(HaveOccurred())
+
 		m = &volumeMounter{
 			podIsolationDetector: &mockIsolationDetector{},
 			mountRecords:         make(map[types.UID]*vmiMountTargetRecord),
 			mountStateDir:        tempDir,
 			skipSafetyCheck:      true,
+			hotplugDiskManager:   hotplugdisk.NewHotplugDiskWithOptions(tempDir),
 		}
 
 		deviceBasePath = func(sourceUID types.UID) string {
@@ -816,6 +850,11 @@ var _ = Describe("HotplugVolume volumes", func() {
 		cgroupsBasePath = func() string {
 			return tempDir
 		}
+		isolationDetector = func(path string) isolation.PodIsolationDetector {
+			return &mockIsolationDetector{
+				pid: 1,
+			}
+		}
 	})
 
 	AfterEach(func() {
@@ -829,49 +868,51 @@ var _ = Describe("HotplugVolume volumes", func() {
 		cgroupsBasePath = orgCgroupsBasePath
 		mknodCommand = orgMknodCommand
 		isBlockDevice = orgIsBlockDevice
+		findMntByVolume = orgFindMntByVolume
 	})
 
 	It("mount and umount should work for filesystem volumes", func() {
-		blockSourcePodUID := types.UID("klmno")
-		fsSourcePodUID := types.UID("fghij")
+		sourcePodUID := types.UID("klmno")
 		volumeStatuses := make([]v1.VolumeStatus, 0)
 		volumeStatuses = append(volumeStatuses, v1.VolumeStatus{
 			Name: "permanent",
 		})
+		block := k8sv1.PersistentVolumeBlock
+		fs := k8sv1.PersistentVolumeFilesystem
 		volumeStatuses = append(volumeStatuses, v1.VolumeStatus{
 			Name: "filesystemvolume",
+			PersistentVolumeClaimInfo: &v1.PersistentVolumeClaimInfo{
+				VolumeMode: &fs,
+			},
 			HotplugVolume: &v1.HotplugVolumeStatus{
-				AttachPodName: "fs-pod",
-				AttachPodUID:  fsSourcePodUID,
+				AttachPodName: "pod",
+				AttachPodUID:  sourcePodUID,
 			},
 		})
 		volumeStatuses = append(volumeStatuses, v1.VolumeStatus{
 			Name: "blockvolume",
+			PersistentVolumeClaimInfo: &v1.PersistentVolumeClaimInfo{
+				VolumeMode: &block,
+			},
 			HotplugVolume: &v1.HotplugVolumeStatus{
-				AttachPodName: "block-pod",
-				AttachPodUID:  blockSourcePodUID,
+				AttachPodName: "pod",
+				AttachPodUID:  sourcePodUID,
 			},
 		})
-		isBlockDevice = func(path string) (bool, error) {
-			log.DefaultLogger().Infof("Checking isBlockDevice for %s", path)
-			if strings.Contains(path, string(blockSourcePodUID)) {
-				return true, nil
-			}
-			log.DefaultLogger().Info("Not a block device")
-			return false, fmt.Errorf("Not a block device")
-		}
 		vmi.Status.VolumeStatus = volumeStatuses
 		deviceBasePath = func(sourceUID types.UID) string {
 			return filepath.Join(tempDir, string(sourceUID), "volumeDevices")
 		}
-		blockDevicePath := filepath.Join(tempDir, string(blockSourcePodUID), "volumeDevices")
-		fileSystemPath := filepath.Join(tempDir, string(fsSourcePodUID), "volumes")
+		blockDevicePath := filepath.Join(tempDir, string(sourcePodUID), "volumeDevices", "blockvolume")
+		fileSystemPath := filepath.Join(tempDir, string(sourcePodUID), "volumes", "disk.img")
+		By(fmt.Sprintf("Creating block path: %s", blockDevicePath))
 		err = os.MkdirAll(blockDevicePath, 0755)
 		Expect(err).ToNot(HaveOccurred())
+		By(fmt.Sprintf("Creating filesystem path: %s", fileSystemPath))
 		err = os.MkdirAll(fileSystemPath, 0755)
 		Expect(err).ToNot(HaveOccurred())
 
-		deviceFile := filepath.Join(blockDevicePath, "file")
+		deviceFile := filepath.Join(blockDevicePath, "blockvolumefile")
 		slicePath := "slice"
 		m.podIsolationDetector = &mockIsolationDetector{
 			slice: slicePath,
@@ -891,23 +932,26 @@ var _ = Describe("HotplugVolume volumes", func() {
 		Expect(err).ToNot(HaveOccurred())
 
 		sourcePodBasePath = func(podUID types.UID) string {
-			if podUID == blockSourcePodUID {
+			if podUID == sourcePodUID {
 				return blockDevicePath
 			}
 			return fileSystemPath
 		}
+		findMntByVolume = func(volumeName string, pid int) ([]byte, error) {
+			return []byte(fmt.Sprintf(findmntByVolumeRes, "filesystemvolume", fileSystemPath)), nil
+		}
+
 		diskFile := filepath.Join(fileSystemPath, "disk.img")
 		_, err = os.Create(diskFile)
 		Expect(err).ToNot(HaveOccurred())
-		hotplugdisk.SetKubeletPodsDirectory(tempDir)
-		targetPodPath := filepath.Join(tempDir, string(m.findVirtlauncherUID(vmi)), "volumes/kubernetes.io~empty-dir/hotplug-disks")
-		err = os.MkdirAll(targetPodPath, 0755)
-		fileSystemVolume := filepath.Join(tempDir, "/abcd/volumes/kubernetes.io~empty-dir/hotplug-disks/filesystemvolume")
-		blockVolume := filepath.Join(tempDir, "/abcd/volumes/kubernetes.io~empty-dir/hotplug-disks/blockvolume")
-		Expect(err).ToNot(HaveOccurred())
-		targetFilePath := filepath.Join(targetPodPath, "filesystemvolume")
+		mknodCommand = func(deviceName string, major, minor int64, blockDevicePermissions string) ([]byte, error) {
+			Expect(os.MkdirAll(deviceName, 0755)).To(Succeed())
+			return []byte("Yay"), nil
+		}
+		blockVolume := filepath.Join(targetPodPath, "blockvolume")
+		targetFilePath := filepath.Join(targetPodPath, "filesystemvolume.img")
 		mountCommand = func(sourcePath, targetPath string) ([]byte, error) {
-			Expect(sourcePath).To(Equal(fileSystemPath))
+			Expect(sourcePath).To(Equal(filepath.Join(fileSystemPath, "disk.img")))
 			Expect(targetPath).To(Equal(targetFilePath))
 			return []byte("Success"), nil
 		}
@@ -917,7 +961,7 @@ var _ = Describe("HotplugVolume volumes", func() {
 		record := &vmiMountTargetRecord{
 			MountTargetEntries: []vmiMountTargetEntry{
 				{
-					TargetFile: fileSystemVolume,
+					TargetFile: targetFilePath,
 				},
 				{
 					TargetFile: blockVolume,
@@ -929,7 +973,7 @@ var _ = Describe("HotplugVolume volumes", func() {
 		bytes, err := ioutil.ReadFile(filepath.Join(tempDir, string(vmi.UID)))
 		Expect(err).ToNot(HaveOccurred())
 		Expect(bytes).To(Equal(expectedBytes))
-		_, err = os.Stat(fileSystemVolume)
+		_, err = os.Stat(targetFilePath)
 		Expect(err).ToNot(HaveOccurred())
 		_, err = os.Stat(blockVolume)
 		Expect(err).ToNot(HaveOccurred())
@@ -943,8 +987,8 @@ var _ = Describe("HotplugVolume volumes", func() {
 		Expect(err).ToNot(HaveOccurred())
 		_, err = ioutil.ReadFile(filepath.Join(tempDir, string(vmi.UID)))
 		Expect(err).To(HaveOccurred(), "record file still exists %s", filepath.Join(tempDir, string(vmi.UID)))
-		_, err = os.Stat(fileSystemVolume)
-		Expect(err).To(HaveOccurred(), "filesystem volume file still exists %s", fileSystemVolume)
+		_, err = os.Stat(targetFilePath)
+		Expect(err).To(HaveOccurred(), "filesystem volume file still exists %s", targetFilePath)
 		_, err = os.Stat(blockVolume)
 		Expect(err).To(HaveOccurred(), "block device volume still exists %s", blockVolume)
 	})
@@ -959,43 +1003,50 @@ var _ = Describe("HotplugVolume volumes", func() {
 	})
 
 	It("unmountAll should cleanup regardless of vmi volumestatuses", func() {
-		blockSourcePodUID := types.UID("klmno")
-		fsSourcePodUID := types.UID("fghij")
+		log.DefaultLogger().Infof("tempdir: %s", tempDir)
+		sourcePodUID := types.UID("klmno")
 		volumeStatuses := make([]v1.VolumeStatus, 0)
+		mknodCommand = func(deviceName string, major, minor int64, blockDevicePermissions string) ([]byte, error) {
+			return []byte("Yay"), nil
+		}
 		volumeStatuses = append(volumeStatuses, v1.VolumeStatus{
 			Name: "permanent",
 		})
+		block := k8sv1.PersistentVolumeBlock
+		fs := k8sv1.PersistentVolumeFilesystem
 		volumeStatuses = append(volumeStatuses, v1.VolumeStatus{
 			Name: "filesystemvolume",
+			PersistentVolumeClaimInfo: &v1.PersistentVolumeClaimInfo{
+				VolumeMode: &fs,
+			},
 			HotplugVolume: &v1.HotplugVolumeStatus{
-				AttachPodName: "fs-pod",
-				AttachPodUID:  fsSourcePodUID,
+				AttachPodName: "pod",
+				AttachPodUID:  sourcePodUID,
 			},
 		})
 		volumeStatuses = append(volumeStatuses, v1.VolumeStatus{
 			Name: "blockvolume",
+			PersistentVolumeClaimInfo: &v1.PersistentVolumeClaimInfo{
+				VolumeMode: &block,
+			},
 			HotplugVolume: &v1.HotplugVolumeStatus{
-				AttachPodName: "block-pod",
-				AttachPodUID:  blockSourcePodUID,
+				AttachPodName: "pod",
+				AttachPodUID:  sourcePodUID,
 			},
 		})
-		isBlockDevice = func(path string) (bool, error) {
-			if strings.Contains(path, string(blockSourcePodUID)) {
-				return true, nil
-			}
-			return false, fmt.Errorf("Not a block device")
-		}
-
 		vmi.Status.VolumeStatus = volumeStatuses
 		deviceBasePath = func(sourceUID types.UID) string {
 			return filepath.Join(tempDir, string(sourceUID), "volumeDevices")
 		}
-		blockDevicePath := filepath.Join(tempDir, string(blockSourcePodUID), "volumeDevices")
-		fileSystemPath := filepath.Join(tempDir, string(fsSourcePodUID), "volumes")
+		blockDevicePath := filepath.Join(tempDir, string(sourcePodUID), "volumeDevices", "blockvolume")
+		fileSystemPath := filepath.Join(tempDir, string(sourcePodUID), "volumes")
 		err = os.MkdirAll(blockDevicePath, 0755)
 		Expect(err).ToNot(HaveOccurred())
 		err = os.MkdirAll(fileSystemPath, 0755)
 		Expect(err).ToNot(HaveOccurred())
+		findMntByVolume = func(volumeName string, pid int) ([]byte, error) {
+			return []byte(fmt.Sprintf(findmntByVolumeRes, "filesystemvolume", fileSystemPath)), nil
+		}
 
 		deviceFile := filepath.Join(blockDevicePath, "file")
 		slicePath := "slice"
@@ -1015,9 +1066,8 @@ var _ = Describe("HotplugVolume volumes", func() {
 		Expect(err).ToNot(HaveOccurred())
 		err = ioutil.WriteFile(deviceFile, []byte("test"), 0644)
 		Expect(err).ToNot(HaveOccurred())
-
 		sourcePodBasePath = func(podUID types.UID) string {
-			if podUID == blockSourcePodUID {
+			if podUID == sourcePodUID {
 				return blockDevicePath
 			}
 			return fileSystemPath
@@ -1025,15 +1075,10 @@ var _ = Describe("HotplugVolume volumes", func() {
 		diskFile := filepath.Join(fileSystemPath, "disk.img")
 		_, err = os.Create(diskFile)
 		Expect(err).ToNot(HaveOccurred())
-		hotplugdisk.SetKubeletPodsDirectory(tempDir)
-		targetPodPath := filepath.Join(tempDir, string(m.findVirtlauncherUID(vmi)), "volumes/kubernetes.io~empty-dir/hotplug-disks")
-		err = os.MkdirAll(targetPodPath, 0755)
-		fileSystemVolume := filepath.Join(tempDir, "/abcd/volumes/kubernetes.io~empty-dir/hotplug-disks/filesystemvolume")
-		blockVolume := filepath.Join(tempDir, "/abcd/volumes/kubernetes.io~empty-dir/hotplug-disks/blockvolume")
-		Expect(err).ToNot(HaveOccurred())
-		targetFilePath := filepath.Join(targetPodPath, "filesystemvolume")
+		blockVolume := filepath.Join(targetPodPath, "blockvolume")
+		targetFilePath := filepath.Join(targetPodPath, "filesystemvolume.img")
 		mountCommand = func(sourcePath, targetPath string) ([]byte, error) {
-			Expect(sourcePath).To(Equal(fileSystemPath))
+			Expect(sourcePath).To(Equal(filepath.Join(fileSystemPath, "disk.img")))
 			Expect(targetPath).To(Equal(targetFilePath))
 			return []byte("Success"), nil
 		}
@@ -1043,7 +1088,7 @@ var _ = Describe("HotplugVolume volumes", func() {
 		record := &vmiMountTargetRecord{
 			MountTargetEntries: []vmiMountTargetEntry{
 				{
-					TargetFile: fileSystemVolume,
+					TargetFile: targetFilePath,
 				},
 				{
 					TargetFile: blockVolume,
@@ -1055,17 +1100,15 @@ var _ = Describe("HotplugVolume volumes", func() {
 		bytes, err := ioutil.ReadFile(filepath.Join(tempDir, string(vmi.UID)))
 		Expect(err).ToNot(HaveOccurred())
 		Expect(bytes).To(Equal(expectedBytes))
-		_, err = os.Stat(fileSystemVolume)
-		Expect(err).ToNot(HaveOccurred())
-		_, err = os.Stat(blockVolume)
+		_, err = os.Stat(targetFilePath)
 		Expect(err).ToNot(HaveOccurred())
 
 		err = m.UnmountAll(vmi)
 		Expect(err).ToNot(HaveOccurred())
 		_, err = ioutil.ReadFile(filepath.Join(tempDir, string(vmi.UID)))
 		Expect(err).To(HaveOccurred(), "record file still exists %s", filepath.Join(tempDir, string(vmi.UID)))
-		_, err = os.Stat(fileSystemVolume)
-		Expect(err).To(HaveOccurred(), "filesystem volume file still exists %s", fileSystemVolume)
+		_, err = os.Stat(targetFilePath)
+		Expect(err).To(HaveOccurred(), "filesystem volume file still exists %s", targetFilePath)
 		_, err = os.Stat(blockVolume)
 		Expect(err).To(HaveOccurred(), "block device volume still exists %s", blockVolume)
 	})
@@ -1073,18 +1116,19 @@ var _ = Describe("HotplugVolume volumes", func() {
 
 type mockIsolationDetector struct {
 	pid        int
+	ppid       int
 	slice      string
 	controller []string
 	err        error
 }
 
 func (i *mockIsolationDetector) Detect(_ *v1.VirtualMachineInstance) (isolation.IsolationResult, error) {
-	return isolation.NewIsolationResult(i.pid, i.slice, i.controller), i.err
+	return isolation.NewIsolationResult(i.pid, i.ppid, i.slice, i.controller), i.err
 }
 
 func (i *mockIsolationDetector) DetectForSocket(_ *v1.VirtualMachineInstance, _ string) (isolation.IsolationResult, error) {
 	if i.pid == 1 {
-		return isolation.NewIsolationResult(i.pid, tempDir, []string{}), nil
+		return isolation.NewIsolationResult(i.pid, i.ppid, tempDir, []string{}), nil
 	}
 	return nil, fmt.Errorf("isolation error")
 }
