@@ -9,6 +9,8 @@ import (
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 
+	"kubevirt.io/kubevirt/tests/util"
+
 	expect "github.com/google/goexpect"
 	admissionregistrationv1 "k8s.io/api/admissionregistration/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -21,7 +23,7 @@ import (
 	v1 "kubevirt.io/client-go/api/v1"
 	snapshotv1 "kubevirt.io/client-go/apis/snapshot/v1alpha1"
 	"kubevirt.io/client-go/kubecli"
-	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1alpha1"
+	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"kubevirt.io/kubevirt/tests"
 	"kubevirt.io/kubevirt/tests/console"
 	cd "kubevirt.io/kubevirt/tests/containerdisk"
@@ -37,7 +39,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 
 	BeforeEach(func() {
 		virtClient, err = kubecli.GetKubevirtClient()
-		tests.PanicOnError(err)
+		util.PanicOnError(err)
 	})
 
 	createRestoreDef := func(vm *v1.VirtualMachine, snapshotName string) *snapshotv1.VirtualMachineRestore {
@@ -77,7 +79,9 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 		Eventually(func() bool {
 			s, err = virtClient.VirtualMachineSnapshot(s.Namespace).Get(context.Background(), s.Name, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
-			return s.Status != nil && s.Status.ReadyToUse != nil && *s.Status.ReadyToUse
+			vm, err = virtClient.VirtualMachine(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			return s.Status != nil && s.Status.ReadyToUse != nil && *s.Status.ReadyToUse && vm.Status.SnapshotInProgress == nil
 		}, 180*time.Second, time.Second).Should(BeTrue())
 
 		return s
@@ -125,7 +129,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 	waitDVReady := func(dv *cdiv1.DataVolume) *cdiv1.DataVolume {
 		Eventually(func() bool {
 			var err error
-			dv, err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(dv.Namespace).Get(context.Background(), dv.Name, metav1.GetOptions{})
+			dv, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(dv.Namespace).Get(context.Background(), dv.Name, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			return dv.Status.Phase == cdiv1.Succeeded
 		}, 180*time.Second, time.Second).Should(BeTrue())
@@ -192,7 +196,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 
 		Context("and no snapshot", func() {
 			It("[test_id:5255]should reject restore", func() {
-				vm, err = virtClient.VirtualMachine(tests.NamespaceTestDefault).Create(vm)
+				vm, err = virtClient.VirtualMachine(util.NamespaceTestDefault).Create(vm)
 				Expect(err).ToNot(HaveOccurred())
 				restore := createRestoreDef(vm, "foobar")
 
@@ -215,7 +219,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 			It("should successfully restore", func() {
 				vm.Spec.Running = nil
 				vm.Spec.RunStrategy = &runStrategyHalted
-				vm, err = virtClient.VirtualMachine(tests.NamespaceTestDefault).Create(vm)
+				vm, err = virtClient.VirtualMachine(util.NamespaceTestDefault).Create(vm)
 				Expect(err).ToNot(HaveOccurred())
 				snapshot = createSnapshot(vm)
 
@@ -236,7 +240,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 			var webhook *admissionregistrationv1.ValidatingWebhookConfiguration
 
 			BeforeEach(func() {
-				vm, err = virtClient.VirtualMachine(tests.NamespaceTestDefault).Create(vm)
+				vm, err = virtClient.VirtualMachine(util.NamespaceTestDefault).Create(vm)
 				Expect(err).ToNot(HaveOccurred())
 				snapshot = createSnapshot(vm)
 			})
@@ -256,6 +260,10 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 					vm, err = virtClient.VirtualMachine(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
 					Expect(err).ToNot(HaveOccurred())
 
+					if vm.Status.SnapshotInProgress != nil {
+						return false
+					}
+
 					origSpec = vm.Spec.DeepCopy()
 					Expect(origSpec.Template.Spec.Domain.Resources.Requests[corev1.ResourceMemory]).To(Equal(resource.MustParse("128Mi")))
 
@@ -324,7 +332,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 							}},
 							ClientConfig: admissionregistrationv1.WebhookClientConfig{
 								Service: &admissionregistrationv1.ServiceReference{
-									Namespace: tests.NamespaceTestDefault,
+									Namespace: util.NamespaceTestDefault,
 									Name:      "nonexistant",
 									Path:      &whPath,
 								},
@@ -377,13 +385,14 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 		})
 	})
 
-	Context("rook-ceph", func() {
+	Context("[rook-ceph]", func() {
 		Context("With a more complicated VM", func() {
 			var (
 				vm                   *v1.VirtualMachine
 				vmi                  *v1.VirtualMachineInstance
 				snapshot             *snapshotv1.VirtualMachineSnapshot
 				restore              *snapshotv1.VirtualMachineRestore
+				webhook              *admissionregistrationv1.ValidatingWebhookConfiguration
 				snapshotStorageClass string
 			)
 
@@ -408,11 +417,15 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 				if restore != nil {
 					deleteRestore(restore)
 				}
+
+				if webhook != nil {
+					deleteWebhook(webhook)
+				}
 			})
 
-			doRestore := func(device string, onlineSnapshot bool) {
+			doRestore := func(device string, login console.LoginToFactory, onlineSnapshot bool) {
 				By("creating 'message with initial value")
-				Expect(libnet.WithIPv6(console.LoginToCirros)(vmi)).To(Succeed())
+				Expect(libnet.WithIPv6(login)(vmi)).To(Succeed())
 
 				var batch []expect.Batcher
 				if device != "" {
@@ -461,28 +474,28 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 				By("creating snapshot")
 				snapshot = createSnapshot(vm)
 
+				batch = nil
 				if !onlineSnapshot {
 					By("Starting VM")
 					vm = tests.StartVirtualMachine(vm)
+					vmi, err = virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+					Expect(err).ToNot(HaveOccurred())
+
+					Expect(libnet.WithIPv6(login)(vmi)).To(Succeed())
+
+					if device != "" {
+						batch = append(batch, []expect.Batcher{
+							&expect.BSnd{S: "sudo mkdir -p /test\n"},
+							&expect.BExp{R: console.PromptExpression},
+							&expect.BSnd{S: fmt.Sprintf("sudo mount %s /test \n", device)},
+							&expect.BExp{R: console.PromptExpression},
+							&expect.BSnd{S: "echo $?\n"},
+							&expect.BExp{R: console.RetValue("0")},
+						}...)
+					}
 				}
-				vmi, err = virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
 
 				By("updating message")
-				Expect(libnet.WithIPv6(console.LoginToCirros)(vmi)).To(Succeed())
-
-				batch = nil
-
-				if device != "" {
-					batch = append(batch, []expect.Batcher{
-						&expect.BSnd{S: "sudo mkdir -p /test\n"},
-						&expect.BExp{R: console.PromptExpression},
-						&expect.BSnd{S: fmt.Sprintf("sudo mount %s /test \n", device)},
-						&expect.BExp{R: console.PromptExpression},
-						&expect.BSnd{S: "echo $?\n"},
-						&expect.BExp{R: console.RetValue("0")},
-					}...)
-				}
 
 				batch = append(batch, []expect.Batcher{
 					&expect.BSnd{S: "sudo mkdir -p /test/data\n"},
@@ -526,7 +539,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 				Expect(err).ToNot(HaveOccurred())
 
 				By("Verifying original file contents")
-				Expect(libnet.WithIPv6(console.LoginToCirros)(vmi)).To(Succeed())
+				Expect(libnet.WithIPv6(login)(vmi)).To(Succeed())
 
 				batch = nil
 
@@ -560,7 +573,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 			It("[test_id:5259]should restore a vm multiple from the same snapshot", func() {
 				vm, vmi = createAndStartVM(tests.NewRandomVMWithDataVolumeAndUserDataInStorageClass(
 					tests.GetUrl(tests.CirrosHttpUrl),
-					tests.NamespaceTestDefault,
+					util.NamespaceTestDefault,
 					"#!/bin/bash\necho 'hello'\n",
 					snapshotStorageClass,
 				))
@@ -589,25 +602,25 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 			It("[test_id:5260]should restore a vm that boots from a datavolumetemplate", func() {
 				vm, vmi = createAndStartVM(tests.NewRandomVMWithDataVolumeAndUserDataInStorageClass(
 					tests.GetUrl(tests.CirrosHttpUrl),
-					tests.NamespaceTestDefault,
+					util.NamespaceTestDefault,
 					"#!/bin/bash\necho 'hello'\n",
 					snapshotStorageClass,
 				))
 
 				originalDVName := vm.Spec.DataVolumeTemplates[0].Name
 
-				doRestore("", false)
+				doRestore("", console.LoginToCirros, false)
 				Expect(restore.Status.DeletedDataVolumes).To(HaveLen(1))
 				Expect(restore.Status.DeletedDataVolumes).To(ContainElement(originalDVName))
 
-				_, err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(vm.Namespace).Get(context.Background(), originalDVName, metav1.GetOptions{})
+				_, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(vm.Namespace).Get(context.Background(), originalDVName, metav1.GetOptions{})
 				Expect(errors.IsNotFound(err)).To(BeTrue())
 			})
 
 			It("[QUARANTINE][test_id:5261]should restore a vm that boots from a datavolume (not template)", func() {
 				vm = tests.NewRandomVMWithDataVolumeAndUserDataInStorageClass(
 					tests.GetUrl(tests.CirrosHttpUrl),
-					tests.NamespaceTestDefault,
+					util.NamespaceTestDefault,
 					"#!/bin/bash\necho 'hello'\n",
 					snapshotStorageClass,
 				)
@@ -622,17 +635,17 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 				originalPVCName := dv.Name
 				vm.Spec.DataVolumeTemplates = nil
 
-				dv, err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(vm.Namespace).Create(context.Background(), dv, metav1.CreateOptions{})
+				dv, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(vm.Namespace).Create(context.Background(), dv, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 				dv = waitDVReady(dv)
 
 				vm, vmi = createAndStartVM(vm)
 
-				doRestore("", false)
+				doRestore("", console.LoginToCirros, false)
 
 				Expect(restore.Status.DeletedDataVolumes).To(BeEmpty())
 
-				_, err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(vm.Namespace).Get(context.Background(), dv.Name, metav1.GetOptions{})
+				_, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(vm.Namespace).Get(context.Background(), dv.Name, metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
 				_, err = virtClient.CoreV1().PersistentVolumeClaims(vm.Namespace).Get(context.Background(), originalPVCName, metav1.GetOptions{})
@@ -651,13 +664,13 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 				}
 			})
 
-			It("[test_id:5262]should restore a vm that boots from a PVC", func() {
+			It("[QUARANTINE][test_id:5262]should restore a vm that boots from a PVC", func() {
 				quantity, err := resource.ParseQuantity("1Gi")
 				Expect(err).ToNot(HaveOccurred())
 				pvc := &corev1.PersistentVolumeClaim{
 					ObjectMeta: metav1.ObjectMeta{
 						Name:      "restore-pvc-" + rand.String(12),
-						Namespace: tests.NamespaceTestDefault,
+						Namespace: util.NamespaceTestDefault,
 						Annotations: map[string]string{
 							"cdi.kubevirt.io/storage.import.source":   "http",
 							"cdi.kubevirt.io/storage.import.endpoint": tests.GetUrl(tests.CirrosHttpUrl),
@@ -685,7 +698,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 
 				vm, vmi = createAndStartVM(vm)
 
-				doRestore("", false)
+				doRestore("", console.LoginToCirros, false)
 
 				Expect(restore.Status.DeletedDataVolumes).To(BeEmpty())
 				_, err = virtClient.CoreV1().PersistentVolumeClaims(vm.Namespace).Get(context.Background(), originalPVCName, metav1.GetOptions{})
@@ -704,7 +717,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 				}
 			})
 
-			It("[test_id:5263]should restore a vm with containerdisk and blank datavolume", func() {
+			It("[QUARANTINE][test_id:5263]should restore a vm with containerdisk and blank datavolume", func() {
 				quantity, err := resource.ParseQuantity("1Gi")
 				Expect(err).ToNot(HaveOccurred())
 				vmi = tests.NewRandomVMIWithEphemeralDiskAndUserdata(
@@ -719,7 +732,7 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 							Name: dvName,
 						},
 						Spec: cdiv1.DataVolumeSpec{
-							Source: cdiv1.DataVolumeSource{
+							Source: &cdiv1.DataVolumeSource{
 								Blank: &cdiv1.DataVolumeBlankImage{},
 							},
 							PVC: &corev1.PersistentVolumeClaimSpec{
@@ -753,24 +766,239 @@ var _ = SIGDescribe("[Serial]VirtualMachineRestore Tests", func() {
 
 				vm, vmi = createAndStartVM(vm)
 
-				doRestore("/dev/vdc", false)
+				doRestore("/dev/vdc", console.LoginToCirros, false)
 
 				Expect(restore.Status.DeletedDataVolumes).To(HaveLen(1))
 				Expect(restore.Status.DeletedDataVolumes).To(ContainElement(dvName))
-				_, err = virtClient.CdiClient().CdiV1alpha1().DataVolumes(vm.Namespace).Get(context.Background(), dvName, metav1.GetOptions{})
+				_, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(vm.Namespace).Get(context.Background(), dvName, metav1.GetOptions{})
 				Expect(errors.IsNotFound(err)).To(BeTrue())
 			})
 
-			It("[QUARANTINE][test_id:5264]should restore a vm from an online snapshot", func() {
-				vm, vmi = createAndStartVM(tests.NewRandomVMWithBlockDataVolumeAndUserDataInStorageClass(
+			It("should reject vm start if restore in progress", func() {
+				vm, vmi = createAndStartVM(tests.NewRandomVMWithDataVolumeAndUserDataInStorageClass(
 					tests.GetUrl(tests.CirrosHttpUrl),
-					tests.NamespaceTestDefault,
+					util.NamespaceTestDefault,
 					"#!/bin/bash\necho 'hello'\n",
 					snapshotStorageClass,
 				))
 
-				doRestore("", true)
+				By("Stopping VM")
+				vm = tests.StopVirtualMachine(vm)
 
+				By("creating snapshot")
+				snapshot = createSnapshot(vm)
+
+				fp := admissionregistrationv1.Fail
+				sideEffectNone := admissionregistrationv1.SideEffectClassNone
+				whPath := "/foobar"
+				whName := "dummy-webhook-deny-pvc-create.kubevirt.io"
+				wh := &admissionregistrationv1.ValidatingWebhookConfiguration{
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "temp-webhook-deny-pvc-create",
+					},
+					Webhooks: []admissionregistrationv1.ValidatingWebhook{
+						{
+							Name:                    whName,
+							AdmissionReviewVersions: []string{"v1", "v1beta1"},
+							FailurePolicy:           &fp,
+							SideEffects:             &sideEffectNone,
+							Rules: []admissionregistrationv1.RuleWithOperations{{
+								Operations: []admissionregistrationv1.OperationType{
+									admissionregistrationv1.Create,
+								},
+								Rule: admissionregistrationv1.Rule{
+									APIGroups:   []string{""},
+									APIVersions: v1.ApiSupportedWebhookVersions,
+									Resources:   []string{"persistentvolumeclaims"},
+								},
+							}},
+							ClientConfig: admissionregistrationv1.WebhookClientConfig{
+								Service: &admissionregistrationv1.ServiceReference{
+									Namespace: util.NamespaceTestDefault,
+									Name:      "nonexistant",
+									Path:      &whPath,
+								},
+							},
+						},
+					},
+				}
+				wh, err := virtClient.AdmissionregistrationV1().ValidatingWebhookConfigurations().Create(context.Background(), wh, metav1.CreateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				webhook = wh
+
+				restore := createRestoreDef(vm, snapshot.Name)
+
+				restore, err = virtClient.VirtualMachineRestore(vm.Namespace).Create(context.Background(), restore, metav1.CreateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				Eventually(func() bool {
+					restore, err = virtClient.VirtualMachineRestore(restore.Namespace).Get(context.Background(), restore.Name, metav1.GetOptions{})
+					Expect(err).ToNot(HaveOccurred())
+					updatedVM, err := virtClient.VirtualMachine(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+					Expect(err).ToNot(HaveOccurred())
+					return restore.Status != nil &&
+						len(restore.Status.Conditions) == 2 &&
+						restore.Status.Conditions[0].Status == corev1.ConditionFalse &&
+						restore.Status.Conditions[1].Status == corev1.ConditionFalse &&
+						strings.Contains(restore.Status.Conditions[0].Reason, whName) &&
+						strings.Contains(restore.Status.Conditions[1].Reason, whName) &&
+						updatedVM.Status.RestoreInProgress != nil &&
+						*updatedVM.Status.RestoreInProgress == restore.Name
+				}, 180*time.Second, 3*time.Second).Should(BeTrue())
+
+				running := true
+				updatedVM, err := virtClient.VirtualMachine(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				updatedVM.Spec.Running = &running
+				_, err = virtClient.VirtualMachine(updatedVM.Namespace).Update(updatedVM)
+				Expect(err).To(HaveOccurred())
+				Expect(err.Error()).To(ContainSubstring(fmt.Sprintf("Cannot start VM until restore %q completes", restore.Name)))
+
+				deleteWebhook(webhook)
+				webhook = nil
+
+				restore = waitRestoreComplete(restore, vm)
+
+				Eventually(func() bool {
+					updatedVM, err := virtClient.VirtualMachine(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+					Expect(err).ToNot(HaveOccurred())
+					return updatedVM.Status.RestoreInProgress == nil
+				}, 30*time.Second, 3*time.Second).Should(BeTrue())
+
+				vm = tests.StartVirtualMachine(vm)
+				deleteRestore(restore)
+			})
+
+			It("[QUARANTINE][test_id:6053]should restore a vm from an online snapshot", func() {
+				vm, vmi = createAndStartVM(tests.NewRandomVMWithDataVolumeAndUserDataInStorageClass(
+					tests.GetUrl(tests.CirrosHttpUrl),
+					util.NamespaceTestDefault,
+					"#!/bin/bash\necho 'hello'\n",
+					snapshotStorageClass,
+				))
+
+				doRestore("", console.LoginToCirros, true)
+
+			})
+
+			It("[test_id:6766]should restore a vm from an online snapshot with guest agent", func() {
+				quantity, err := resource.ParseQuantity("1Gi")
+				Expect(err).ToNot(HaveOccurred())
+				vmi = tests.NewRandomFedoraVMIWithGuestAgent()
+				vmi.Namespace = util.NamespaceTestDefault
+				vm = tests.NewRandomVirtualMachine(vmi, false)
+				dvName := "dv-" + vm.Name
+				vm.Spec.DataVolumeTemplates = []v1.DataVolumeTemplateSpec{
+					{
+						ObjectMeta: metav1.ObjectMeta{
+							Name: dvName,
+						},
+						Spec: cdiv1.DataVolumeSpec{
+							Source: &cdiv1.DataVolumeSource{
+								Blank: &cdiv1.DataVolumeBlankImage{},
+							},
+							PVC: &corev1.PersistentVolumeClaimSpec{
+								AccessModes: []corev1.PersistentVolumeAccessMode{corev1.ReadWriteOnce},
+								Resources: corev1.ResourceRequirements{
+									Requests: corev1.ResourceList{
+										"storage": quantity,
+									},
+								},
+								StorageClassName: &snapshotStorageClass,
+							},
+						},
+					},
+				}
+				vm.Spec.Template.Spec.Domain.Devices.Disks = append(vm.Spec.Template.Spec.Domain.Devices.Disks, v1.Disk{
+					Name: "blank",
+					DiskDevice: v1.DiskDevice{
+						Disk: &v1.DiskTarget{
+							Bus: "virtio",
+						},
+					},
+				})
+				vm.Spec.Template.Spec.Volumes = append(vm.Spec.Template.Spec.Volumes, v1.Volume{
+					Name: "blank",
+					VolumeSource: v1.VolumeSource{
+						DataVolume: &v1.DataVolumeSource{
+							Name: "dv-" + vm.Name,
+						},
+					},
+				})
+
+				vm, vmi = createAndStartVM(vm)
+				tests.WaitForSuccessfulVMIStartWithTimeout(vmi, 300)
+				tests.WaitAgentConnected(virtClient, vmi)
+
+				doRestore("/dev/vdc", console.LoginToFedora, true)
+
+			})
+
+			It("[test_id:6836]should restore an online vm snapshot that boots from a datavolumetemplate with guest agent", func() {
+				vm, vmi = createAndStartVM(tests.NewRandomVMWithDataVolumeWithRegistryImport(
+					cd.DataVolumeImportUrlForContainerDisk(cd.ContainerDiskFedoraTestTooling),
+					util.NamespaceTestDefault,
+					snapshotStorageClass,
+					corev1.ReadWriteOnce))
+				tests.WaitAgentConnected(virtClient, vmi)
+				Expect(libnet.WithIPv6(console.LoginToFedora)(vmi)).To(Succeed())
+
+				originalDVName := vm.Spec.DataVolumeTemplates[0].Name
+
+				doRestore("", console.LoginToFedora, true)
+				Expect(restore.Status.DeletedDataVolumes).To(HaveLen(1))
+				Expect(restore.Status.DeletedDataVolumes).To(ContainElement(originalDVName))
+
+				_, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(vm.Namespace).Get(context.Background(), originalDVName, metav1.GetOptions{})
+				Expect(errors.IsNotFound(err)).To(BeTrue())
+			})
+
+			It("should restore vm spec at startup without new changes", func() {
+				vm, vmi = createAndStartVM(tests.NewRandomVMWithDataVolumeWithRegistryImport(
+					cd.DataVolumeImportUrlForContainerDisk(cd.ContainerDiskFedoraTestTooling),
+					util.NamespaceTestDefault,
+					snapshotStorageClass,
+					corev1.ReadWriteOnce))
+				tests.WaitAgentConnected(virtClient, vmi)
+				Expect(libnet.WithIPv6(console.LoginToFedora)(vmi)).To(Succeed())
+
+				By("Updating the VM template spec")
+				initialMemory := vmi.Spec.Domain.Resources.Requests[corev1.ResourceMemory]
+				newMemory := resource.MustParse("2Gi")
+				Expect(newMemory).ToNot(Equal(initialMemory))
+
+				newVM, err := virtClient.VirtualMachine(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				updatedVM := newVM.DeepCopy()
+				updatedVM.Spec.Template.Spec.Domain.Resources.Requests = corev1.ResourceList{
+					corev1.ResourceMemory: newMemory,
+				}
+				updatedVM, err = virtClient.VirtualMachine(updatedVM.Namespace).Update(updatedVM)
+				Expect(err).ToNot(HaveOccurred())
+
+				By("creating snapshot")
+				snapshot = createSnapshot(vm)
+
+				newVM = tests.StopVirtualMachine(updatedVM)
+				newVM = tests.StartVirtualMachine(newVM)
+				vmi, err = virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				Expect(vmi.Spec.Domain.Resources.Requests[corev1.ResourceMemory]).To(Equal(newMemory))
+
+				newVM = tests.StopVirtualMachine(newVM)
+
+				By("Restoring VM")
+				restore = createRestoreDef(newVM, snapshot.Name)
+				restore, err = virtClient.VirtualMachineRestore(vm.Namespace).Create(context.Background(), restore, metav1.CreateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				restore = waitRestoreComplete(restore, newVM)
+				Expect(restore.Status.Restores).To(HaveLen(1))
+
+				newVM = tests.StartVirtualMachine(newVM)
+				vmi, err = virtClient.VirtualMachineInstance(vm.Namespace).Get(vm.Name, &metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+				Expect(vmi.Spec.Domain.Resources.Requests[corev1.ResourceMemory]).To(Equal(initialMemory))
 			})
 
 		})
