@@ -8,14 +8,15 @@ import (
 	"strings"
 	"time"
 
-	v12 "kubevirt.io/client-go/api/v1"
-
 	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
+	v12 "kubevirt.io/client-go/api/v1"
 	"kubevirt.io/client-go/kubecli"
-
+	virtctlpause "kubevirt.io/kubevirt/pkg/virtctl/pause"
+	virtctlvm "kubevirt.io/kubevirt/pkg/virtctl/vm"
 	"kubevirt.io/kubevirt/tests"
 	cd "kubevirt.io/kubevirt/tests/containerdisk"
+	"kubevirt.io/kubevirt/tests/util"
 
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -25,14 +26,20 @@ const (
 	// Define relevant k8s versions
 	relevantk8sVer = "1.16.2"
 
-	// Define a timeout for read opeartions in order to prevent test hanging
-	readTimeout       = 30 * time.Second
-	vmCreationTimeout = 1 * time.Minute
+	// Define a timeout for read operations in order to prevent test hanging
+	statusChangeTimeout = 10 * time.Minute
+	vmCreationTimeout   = 2 * time.Minute
+	vmRecreationTimeout = 5 * time.Minute
+	vmMigrationTimeout  = 5 * time.Minute
 
-	vmAgeRegex = "^[0-9]+[sm]$"
+	vmAgeRegex   = "^[0-9]+[sm]$"
+	vmReadyRegex = "False|True"
 
 	// Define a buffer size to read errors into
 	bufferSize = 1024
+
+	readyConditionFalse = "False"
+	readyConditionTrue  = "True"
 )
 
 // Reads up to buffSize characters from rc
@@ -94,28 +101,44 @@ func readLineWithTimeout(rc io.ReadCloser, timeout time.Duration) (string, error
 
 // Reads VM/VMI status from rc and returns a new status.
 // If oldStatus is non-nil, the function will read status lines until
-// newStatus.running != oldStatus.running or newStatus.phase != oldStatus.phase
+// newStatus.printableStatus != oldStatus.printableStatus or newStatus.phase != oldStatus.phase
 // in order to skip duplicated status lines
 func readNewStatus(rc io.ReadCloser, oldStatus []string, timeout time.Duration) ([]string, error) {
-	statusLine, err := readLineWithTimeout(rc, timeout)
+	prevStatus := oldStatus
 
-	if err != nil {
-		return nil, err
-	}
+	// Iterate repeatedly until a modified line is found
+	remainingTimeout := timeout
+	for {
+		start := time.Now()
+		statusLine, err := readLineWithTimeout(rc, remainingTimeout)
 
-	newStatus := strings.Fields(statusLine)
+		if err != nil {
+			return nil, fmt.Errorf("error reading new status: '%v', last observed status: %v", err, prevStatus)
+		}
+		newStatus := strings.Fields(statusLine)
 
-	// Skip status line with similar running state for VM or phase state for VMI
-	// newStatus[2] and oldStatus[2] point to the VM running state or VMI phase
-	if oldStatus != nil && len(oldStatus) == len(newStatus) {
-		if len(oldStatus) == 2 {
-			return readNewStatus(rc, newStatus, timeout)
-		} else if len(oldStatus) >= 3 && newStatus[2] == oldStatus[2] {
-			return readNewStatus(rc, newStatus, timeout)
+		prevStatusOrPhase := ""
+		if len(prevStatus) >= 3 {
+			prevStatusOrPhase = prevStatus[2]
+		}
+
+		newStatusOrPhase := ""
+		if len(newStatus) >= 3 {
+			newStatusOrPhase = newStatus[2]
+		}
+
+		// Only return if a change is detected in the printableStatus of a VM,
+		// or the phase of a VMI
+		if prevStatusOrPhase != newStatusOrPhase {
+			return newStatus, nil
+		}
+
+		prevStatus = newStatus
+		remainingTimeout -= time.Now().Sub(start)
+		if remainingTimeout <= 0 {
+			return nil, fmt.Errorf("timeout waiting for new VM/VMI status line")
 		}
 	}
-
-	return newStatus, nil
 }
 
 // Create a command with output/error redirection.
@@ -163,35 +186,35 @@ var _ = Describe("[rfe_id:3423][crit:high][vendor:cnv-qe@redhat.com][level:compo
 
 	BeforeEach(func() {
 		virtCli, err = kubecli.GetKubevirtClient()
-		tests.PanicOnError(err)
+		util.PanicOnError(err)
 
 		tests.SkipIfVersionBelow("Printing format for `kubectl get -w` on custom resources is only relevant for 1.16.2+", relevantk8sVer)
 		tests.BeforeTestCleanup()
 
 		vm = tests.NewRandomVMWithEphemeralDisk(cd.ContainerDiskFor(cd.ContainerDiskCirros))
 		vm, err = virtCli.VirtualMachine(vm.ObjectMeta.Namespace).Create(vm)
-		tests.PanicOnError(err)
+		util.PanicOnError(err)
 
 		By("Making sure kubectl cache is updated to contain vm/vmi resources")
 		Eventually(func() bool {
-			_, getVM, err := tests.CreateCommandWithNS(tests.NamespaceTestDefault, tests.GetK8sCmdClient(), "get", "vm")
-			tests.PanicOnError(err)
-			_, getVMI, err := tests.CreateCommandWithNS(tests.NamespaceTestDefault, tests.GetK8sCmdClient(), "get", "vmi")
-			tests.PanicOnError(err)
+			_, getVM, err := tests.CreateCommandWithNS(util.NamespaceTestDefault, tests.GetK8sCmdClient(), "get", "vm")
+			util.PanicOnError(err)
+			_, getVMI, err := tests.CreateCommandWithNS(util.NamespaceTestDefault, tests.GetK8sCmdClient(), "get", "vmi")
+			util.PanicOnError(err)
 
 			return getVM.Run() == nil && getVMI.Run() == nil
 		}, vmCreationTimeout, 1*time.Millisecond).Should(BeTrue())
 	})
 
 	AfterEach(func() {
-		err := virtCli.VirtualMachine(tests.NamespaceTestDefault).Delete(vm.Name, &v1.DeleteOptions{})
-		tests.PanicOnError(err)
+		err := virtCli.VirtualMachine(util.NamespaceTestDefault).Delete(vm.Name, &v1.DeleteOptions{})
+		util.PanicOnError(err)
 	})
 
-	It("[test_id:3468]Should update vm status with the proper columns using 'kubectl get vm -w'", func() {
+	It("[QUARANTINE][test_id:3468]Should update vm status with the proper columns using 'kubectl get vm -w'", func() {
 		By("Waiting for a VM to be created")
 		Eventually(func() bool {
-			_, err := virtCli.VirtualMachine(tests.NamespaceTestDefault).Get(vm.Name, &v1.GetOptions{})
+			_, err := virtCli.VirtualMachine(util.NamespaceTestDefault).Get(vm.Name, &v1.GetOptions{})
 			return err == nil
 		}, vmCreationTimeout, 1*time.Millisecond).Should(BeTrue())
 
@@ -209,16 +232,88 @@ var _ = Describe("[rfe_id:3423][crit:high][vendor:cnv-qe@redhat.com][level:compo
 		go readFromStderr(stderr)
 
 		// Read column titles
-		titles, err := readNewStatus(stdout, nil, readTimeout)
+		titles, err := readNewStatus(stdout, nil, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(titles).To(Equal([]string{"NAME", "AGE", "VOLUME"}),
+		Expect(titles).To(Equal([]string{"NAME", "AGE", "STATUS", "READY"}),
 			"Output should have the proper columns")
+
+		vmStatus, err := readNewStatus(stdout, nil, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusStopped), readyConditionFalse),
+			"VM should be in the %s status", v12.VirtualMachineStatusStopped)
+
+		By("Starting the VM")
+		vm = tests.StartVirtualMachine(vm)
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusStarting), readyConditionFalse),
+			"VM should be in the %s status", v12.VirtualMachineStatusStarting)
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusRunning), MatchRegexp(vmReadyRegex)),
+			"VM should be in the %s status", v12.VirtualMachineStatusRunning)
+
+		By("Pausing the VirtualMachine")
+		pauseCommand := tests.NewRepeatableVirtctlCommand(virtctlpause.COMMAND_PAUSE, "vm", "--namespace", vm.Namespace, vm.Name)
+		Expect(pauseCommand()).To(Succeed())
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusPaused), readyConditionTrue),
+			"VM should be in the %s status", v12.VirtualMachineStatusPaused)
+
+		By("Unpausing the VirtualMachine")
+		unpauseCommand := tests.NewRepeatableVirtctlCommand(virtctlpause.COMMAND_UNPAUSE, "vm", "--namespace", vm.Namespace, vm.Name)
+		Expect(unpauseCommand()).To(Succeed())
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusRunning), readyConditionTrue),
+			"VM should be in the %s status", v12.VirtualMachineStatusRunning)
+
+		By("Migrating the VirtualMachine")
+
+		// Verify we have more than one scheduleable node
+		virtClient, err := kubecli.GetKubevirtClient()
+		Expect(err).ToNot(HaveOccurred())
+
+		nodes := util.GetAllSchedulableNodes(virtClient)
+		Expect(len(nodes.Items)).To(BeNumerically(">=", 2),
+			"Migration requires at least 2 schedulable nodes")
+
+		migrateCommand := tests.NewRepeatableVirtctlCommand(virtctlvm.COMMAND_MIGRATE, "--namespace", vm.Namespace, vm.Name)
+		Expect(migrateCommand()).To(Succeed())
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, vmMigrationTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusMigrating), readyConditionTrue),
+			"VM should be in the %s status", v12.VirtualMachineStatusMigrating)
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusRunning), readyConditionTrue),
+			"VM should be in the %s status", v12.VirtualMachineStatusRunning)
+
+		By("Stopping the VirtualMachine")
+		vm = tests.StopVirtualMachine(vm)
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusStopping), MatchRegexp(vmReadyRegex)),
+			"VM should be in the %s status", v12.VirtualMachineStatusStopping)
+
+		vmStatus, err = readNewStatus(stdout, vmStatus, statusChangeTimeout)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(vmStatus).To(ConsistOf(vm.Name, MatchRegexp(vmAgeRegex), string(v12.VirtualMachineStatusStopped), MatchRegexp(vmReadyRegex)),
+			"VM should be in the %s status", v12.VirtualMachineStatusStopped)
 	})
 
 	It("[test_id:3466]Should update vmi status with the proper columns using 'kubectl get vmi -w'", func() {
 		By("Waiting for a VM to be created")
 		Eventually(func() bool {
-			_, err := virtCli.VirtualMachine(tests.NamespaceTestDefault).Get(vm.Name, &v1.GetOptions{})
+			_, err := virtCli.VirtualMachine(util.NamespaceTestDefault).Get(vm.Name, &v1.GetOptions{})
 			return err == nil
 		}, vmCreationTimeout, 1*time.Second).Should(BeTrue())
 
@@ -240,13 +335,13 @@ var _ = Describe("[rfe_id:3423][crit:high][vendor:cnv-qe@redhat.com][level:compo
 		go readFromStderr(stderr)
 
 		// Read the column titles
-		titles, err := readNewStatus(stdout, nil, readTimeout)
+		titles, err := readNewStatus(stdout, nil, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(titles).To(Equal([]string{"NAME", "AGE", "PHASE", "IP", "NODENAME"}),
+		Expect(titles).To(Equal([]string{"NAME", "AGE", "PHASE", "IP", "NODENAME", "READY"}),
 			"Output should have the proper columns")
 
 		// Read out the guard VMI
-		vmiStatus, err := readNewStatus(stdout, titles, readTimeout)
+		vmiStatus, err := readNewStatus(stdout, nil, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
 
 		// Start a VMI
@@ -255,7 +350,7 @@ var _ = Describe("[rfe_id:3423][crit:high][vendor:cnv-qe@redhat.com][level:compo
 
 		By("Waiting for the VMI to be created")
 		Eventually(func() bool {
-			list, err := virtCli.VirtualMachineInstance(tests.NamespaceTestDefault).List(&v1.ListOptions{})
+			list, err := virtCli.VirtualMachineInstance(util.NamespaceTestDefault).List(&v1.ListOptions{})
 			Expect(err).ToNot(HaveOccurred())
 
 			if len(list.Items) >= 2 {
@@ -272,18 +367,18 @@ var _ = Describe("[rfe_id:3423][crit:high][vendor:cnv-qe@redhat.com][level:compo
 
 		// There might be a second (or more?) guardVmi "running" line in the pipeline... Squashing it (/them) first
 		for vmiStatus[0] == guardVmi.Name {
-			vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+			vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 			Expect(err).ToNot(HaveOccurred())
 		}
 		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Pending)),
 			"VMI should be in the Pending phase")
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Scheduling)),
+		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Scheduling), readyConditionFalse),
 			"VMI should be in the Scheduling phase")
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
 		// "scheduled" lines may or may not contain an IP, parsing only the first 3 fields
 		Expect(len(vmiStatus)).To(BeNumerically(">=", 3), fmt.Sprintf("vmiStatus is missing expected properties %v", vmiStatus))
@@ -291,27 +386,27 @@ var _ = Describe("[rfe_id:3423][crit:high][vendor:cnv-qe@redhat.com][level:compo
 		Expect(vmiStatus[1]).To(MatchRegexp(vmAgeRegex))
 		Expect(vmiStatus[2]).To(Equal(string(v12.Scheduled)), "VMI should be in the Scheduled phase")
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(len(vmiStatus)).To(Equal(5), fmt.Sprintf("vmiStatus is missing expected properties %v", vmiStatus))
+		Expect(len(vmiStatus)).To(Equal(6), fmt.Sprintf("vmiStatus is missing expected properties %v", vmiStatus))
 		Expect(net.ParseIP(vmiStatus[3])).ToNot(BeNil())
-		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Running), vmiStatus[3], vmi.Status.NodeName),
+		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Running), vmiStatus[3], vmi.Status.NodeName, MatchRegexp(vmReadyRegex)),
 			"VMI should be in the Running phase")
 
 		// Restart the VMI
 		err = virtCli.VirtualMachine(vm.ObjectMeta.Namespace).Restart(vm.ObjectMeta.Name)
 		Expect(err).ToNot(HaveOccurred(), "VMI should have been restarted")
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(len(vmiStatus)).To(Equal(5), fmt.Sprintf("vmiStatus is missing expected properties %v", vmiStatus))
+		Expect(len(vmiStatus)).To(Equal(6), fmt.Sprintf("vmiStatus is missing expected properties %v", vmiStatus))
 		Expect(net.ParseIP(vmiStatus[3])).ToNot(BeNil())
-		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Succeeded), vmiStatus[3], vmi.Status.NodeName),
-			"VMI should be in the Failed phase")
+		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Succeeded), vmiStatus[3], vmi.Status.NodeName, MatchRegexp(vmReadyRegex)),
+			"VMI should be in the Succeeded phase")
 
 		By("Waiting for the second VMI to be created")
 		Eventually(func() bool {
-			list, err := virtCli.VirtualMachineInstance(tests.NamespaceTestDefault).List(&v1.ListOptions{})
+			list, err := virtCli.VirtualMachineInstance(util.NamespaceTestDefault).List(&v1.ListOptions{})
 			Expect(err).ToNot(HaveOccurred())
 
 			if len(list.Items) >= 2 {
@@ -326,30 +421,30 @@ var _ = Describe("[rfe_id:3423][crit:high][vendor:cnv-qe@redhat.com][level:compo
 			}
 
 			return false
-		}, vmCreationTimeout, 1*time.Second).Should(BeTrue())
+		}, vmRecreationTimeout, 1*time.Second).Should(BeTrue())
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Pending)),
 			"VMI should be in the Pending phase")
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Scheduling)),
+		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Scheduling), readyConditionFalse),
 			"VMI should be in the Scheduling phase")
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(len(vmiStatus)).To(BeNumerically(">=", 3), fmt.Sprintf("vmiStatus is missing expected properties %v", vmiStatus))
 		Expect(vmiStatus[0]).To(Equal(vmi.Name))
 		Expect(vmiStatus[1]).To(MatchRegexp(vmAgeRegex))
 		Expect(vmiStatus[2]).To(Equal(string(v12.Scheduled)), "VMI should be in the Scheduled phase")
 
-		vmiStatus, err = readNewStatus(stdout, vmiStatus, readTimeout)
+		vmiStatus, err = readNewStatus(stdout, vmiStatus, statusChangeTimeout)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(len(vmiStatus)).To(Equal(5), fmt.Sprintf("vmiStatus is missing expected propertiesL %v", vmiStatus))
+		Expect(len(vmiStatus)).To(Equal(6), fmt.Sprintf("vmiStatus is missing expected propertiesL %v", vmiStatus))
 		Expect(net.ParseIP(vmiStatus[3])).ToNot(BeNil())
-		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Running), vmiStatus[3], vmi.Status.NodeName),
+		Expect(vmiStatus).To(ConsistOf(vmi.Name, MatchRegexp(vmAgeRegex), string(v12.Running), vmiStatus[3], vmi.Status.NodeName, MatchRegexp(vmReadyRegex)),
 			"VMI should be in the Running phase")
 	})
 })
